/* $Id: form.h 117 2008-08-06 14:53:40Z dirk $ */

/* GenLib2 - General Library 2
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef GENLIB2_FORM_INCLUDED
#define GENLIB2_FORM_INCLUDED

#include <string>
#include <iostream>
#include <sstream>

using namespace std;


namespace GenLib2
{

struct Bound_form_num;
struct Bound_form_str;

/** Formating output class.
 *  Formating floating-point numbers, integers, c-strings
 *  and STL strings.
 *  @author Dirk Stallmann
 */
class Form
{
	friend ostream& operator<<(ostream&, const Bound_form_num&);
	friend ostream& operator<<(ostream&, const Bound_form_str&);

public:
	explicit Form(int p=6) : prc(p)
	{
		fmt = ios::fmtflags(0);	// general format
		wdt = 0;	// as wide as necessary
		chr = ' ';	// space character
	}

	Bound_form_num operator()(double d) const;
	Bound_form_num operator()(float f) const;
	Bound_form_num operator()(int i) const;
	Bound_form_num operator()(unsigned int i) const;
	Bound_form_num operator()(long unsigned int i) const;


	Bound_form_str operator()(const char* p) const;
	Bound_form_str operator()(const string& s) const;

	/// set floating-point output format
	Form& scientific() { fmt |= ios::scientific; return *this; }

	/// fixed format
	Form& fixed() { fmt |= ios::fixed; return *this; }

	// general format
	Form& general() { fmt |= ios::fmtflags(0); return *this; }

	Form& uppercase() { fmt |= ios::uppercase; return *this; }
	Form& lowercase() { fmt &= ~ios::uppercase; return *this; }

	/// set precision
	Form& precision(int p) { prc = p; return *this; }

	/// set field width
	Form& width(int w) { wdt = w; return *this; }

	/// set padding character
	Form& fill(char c) { chr = c; return *this; }

	/// show
	Form& showpoint() { fmt |= ios::showpoint; return *this; }

	/// show
	Form& noshowpoint() { fmt &= ~ios::showpoint; return *this; }

	/// sign (+ or -) always be placed before a number
	Form& showpos() { fmt |= ios::showpos; return *this; }

	/// sign is used only for negative numbers (default)
	Form& noshowpos() { fmt &= ~ios::showpos; return *this; }

	/// fieldadjustment on the field boundary
	/// left adjusted (default)
	Form& left() { fmt = ios::left; return *this; }
	/// right adjusted
	Form& right() { fmt = ios::right; return *this; }

	/// internal
	Form& internal() { fmt = ios::internal; return *this; }

private:
	/// precision
	int		prc;

	/// field width
	int		wdt;

	/// format: general, scientific or fixed
	ios::fmtflags	fmt;

	/// fill character
	char	chr;
};

struct Bound_form_num
{
	const Form&		f;
	double			val;

	Bound_form_num(const Form& ff, double d) : f(ff), val(d) { }
	Bound_form_num(const Form& ff, float f) : f(ff), val(double(f)) { }
	Bound_form_num(const Form& ff, int i) : f(ff), val(double(i)) { }
	Bound_form_num(const Form& ff, unsigned int i) : f(ff), val(double(i)) { }
	Bound_form_num(const Form& ff, long unsigned int i) : f(ff), val(double(i)) { }
};

struct Bound_form_str
{
	const Form&		f;
	const char*		str;

	Bound_form_str(const Form& ff, const char* p) : f(ff), str(p) { }
	Bound_form_str(const Form& ff, const string& s) : f(ff), str(s.c_str()) { }
};


// Implementations ...

inline Bound_form_num Form::operator()(double d) const
{
	return Bound_form_num(*this, d);
}


inline Bound_form_num Form::operator()(float f) const
{
	return Bound_form_num(*this, f);
}


inline Bound_form_num Form::operator()(int i) const
{
	return Bound_form_num(*this, i);
}


inline Bound_form_num Form::operator()(unsigned int i) const
{
	return Bound_form_num(*this, i);
}


inline Bound_form_num Form::operator()(long unsigned int i) const
{
	return Bound_form_num(*this, i);
}


inline Bound_form_str Form::operator()(const char* p) const
{
	return Bound_form_str(*this, p);
}


inline Bound_form_str Form::operator()(const string& s) const
{
	return Bound_form_str(*this, s);
}


inline ostream& operator<<(ostream& os, const Bound_form_num& bf)
{
	ostringstream s;

	s.flags(bf.f.fmt);		// set flags
	s.precision(bf.f.prc);	// set precision
	s.width(bf.f.wdt);		// set width
	s << bf.val;			// compose string in s

	return os << s.str();
}


inline ostream& operator<<(ostream& os, const Bound_form_str& bf)
{
	ostringstream s;

	s.flags(bf.f.fmt);		// set flags
	s.width(bf.f.wdt);		// set width
	s << bf.str;			// compose string in s

	return os << s.str();
}


} // namespace GenLib2

#endif /* GENLIB2_FORM_INCLUDED */
