/* $Id: sstransf.h 117 2008-08-06 14:53:40Z dirk $ */

/* GenLib2 - General Library 2
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef GENLIB2_SSTRANSF_INCLUDED
#define GENLIB2_SSTRANSF_INCLUDED

#include "drotmatrix3.h"
 
namespace GenLib2
{

/** Spatial similarity transformation.
 *  Also known as 7-parameter transformation or Helmert transformation.
 *  Transformation between two 3-dimensional cartesian coordinate systems
 *  in the general case by three translations, an orthogonal rotation matrix
 *  and a scale:
 *
 *    \[ r_2 = r_o + s R r_1 \]
 *
 *  and with the inverse transformation:
 *
 *    \[ r_1 = \frac{1}{s} (r_2 - r_o) R^T \]
 *
 *  where
 *
 *  \begin{tabular}{ll}
 *    $r_1 = (X_1, Y_1, Z_1)$	& position vector in system 1\\
 *    $r_2 = (X_2, Y_2, Z_2)$	& position vector in system 2\\
 *    $r_o = (X_o, Y_o, Z_o)$	& translation vector or origin \\
 *    $s$						& scale factor \\
 *    $R$						& rotation matrix. \\
 *  \end{tabular}
 *
 *  The 7 transformation parameters are dervied by a set of points
 *  common to both coordinate systems.
 *
 *  In geodetic applications this transformation is called Bursa-Wolf-model.
 *  See Heck: Rechenverfahren und Auswertemodelle der Landesvermessung
 *  (1980, pp. 48, 345, 378) and Torge: Geodesy (1980, pp. 189).
 *
 *  @author Dirk Stallmann
 */
class SSTransform
{
public:
	/// default constructor with: $s = 1, r_o = 0, R = I$
	SSTransform();

	/// set translation vector
	void set_translation(double dx, double dy, double dz) {
		_dx = dx; _dy = dy; _dz = dz; }

	/// set rotation matrix
	void set_rot_matrix(const DRotMatrix3& rm) { _rot_matrix = rm; }

	/// set scale factor
	void set_scale(double s) { _scale = s; }

	/** set/get scale correction (s = 1 + m)
	 */
	//@{
		// set scale correction
		//void set_scale_corr(double m) { _scale = 1. + m; }

		// get scale correction
		//double get_scale_corr() const { return (_scale - 1.); }
	//@}

	/** set rotation matrix for small rotations (Euler angles).
	 *  Attention: The differential rotations have in the template
	 *  class RotMatrix3 the opposite sign!
	 */
	void set_rot_matrix(double eps_x, double eps_y, double eps_z);

	/** transformation of position vector
	 *  $r_1 \rightarrow r_2$
	 */
	void transform(double x1, double y1, double z1,
		double& x2, double& y2, double& z2) const;

	/** inverse transformation of position vector
	 *  $r_2 \rightarrow r_1$
	 */
	void inv_transform(double x2, double y2, double z2,
		double& x1, double& y1, double& z1) const;

	/** read transformation parameters: scale and translation vector
	 *  Parametrisation of rotation matrix:
	 *   -  for small rotations (Euler angles)
	 */
	bool read(istream& is);

	/// print transformation parameters
	void print(ostream& os) const;

	/// invert transformation parameters
	void invert();

private:
	/// translation in X
	double				_dx;

	/// translation in Y
	double				_dy;

	/// translation in Z
	double				_dz;

	/// rotation matrix
	DRotMatrix3			_rot_matrix;

	/// scale
	double				_scale;
};

} // namespace GenLib2

#endif
