/* $Id: uangle.h 117 2008-08-06 14:53:40Z dirk $ */

/* GenLib2 - General Library 2
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef GENLIB2_UANGLE_INCLUDED
#define GENLIB2_UANGLE_INCLUDED

#include <ostream>
#include "mpc.h"
using namespace std;

namespace GenLib2
{

/** Unit angle class.
 *  Measurements of plane angles.
 *  Supported units: radian, decimal degrees,
 *    decimal minute, decimal second, hours and gon (grad).
 *
 *  @author Dirk Stallmann
 */
class UnitAngle
{
public:
	/// code of unit
	enum ecode
	{
		/// unknown
		UNKNOWN,

		/// without unit is like radian
		NONE,

		/// radian
		RADIAN,

		/// degree
		DEGREE,

		/// hour or angular hour
		HOUR,

		/// minute or angular minute
		MINUTE,

		/// second or angular second
		SECOND,

		/// gon or grad
		GON
	};
	
	/**@name set unit using symbol-string.
	 *
	 *  @param symbol
	 *  @return true = OK, false = error occured
	 */
	bool set(const char* symbol);
	
	/// default constructor
	UnitAngle() { set("rad"); }

	/**@name constructor using symbol-string.
	 *
	 *  @param symbol
	 */
	UnitAngle(const char* symbol) { set(symbol); }
	
	/// destructor
	virtual ~UnitAngle() { }
	
	/**@name get rho.
	 *
	 *  @return scale factor
	 */
	double rho() const { return _rho; }
	
	/**@name get symbol-string.
	 *
	 *  @return symbol
	 */
	const char* symbol() const { return _map[_code].symbol; }
	
	/**@name get code.
	 *
	 *  @param symbol
	 *  @return code
	 */
	static ecode code(const char* symbol);

	/**@name get name.
	 *
	 *  @param code
	 *  @return name
	 */
	static const char* name(int code);

	/**@name get symbol.
	 *
	 *  @param code
	 *  @return symbol
	 */
	static const char* symbol(int code);

	/**@name get rho.
	 *  Get scale factor rho to convert from the actual unit
	 *  into the SI-base unit radian. The division of the angle
	 *  by rho results to the equivalent angle in radian.
	 *
	 *  @param code
	 *  @return scale factor
	 */
	static double rho(int code);

	// conversion from unit base
	//double conv(double v) const { return (v * _scale); }

	// conversion into unit base
	//double ubase(double v) const { return (v / _scale); }

	/// print map
	void print(ostream& os) const;

protected:
	/// code
	ecode	_code;
	
	/// scale factor rho
	double	_rho;
	
	/// map size
	static const int _map_size;

	/// map structure
	struct map_t
	{
		/// code
		int				code;

		/// symbol or short name
		const char*		symbol;

		/// (long) name
		const char*		name;

		/// rho (scale factor)
		double			rho;
	};

	/// map
	static map_t _map[];
};


// non-member functions...

} // namespace GenLib2

#endif /* GENLIB2_UANGLE_INCLUDED */
